# Class taken from http://github.com/railsfactory/rubygems

##
# The Version class processes string versions into comparable values
#
class Version
	include Comparable
	
	attr_accessor :version
	
	NUM_RE = /\s*(\d+(\.\d+)*)*\s*/
	
	##
	# Constructs a version from the supplied string
	#
	# version:: [String] The version string.  Format is digit.digit...
	#
	def initialize(version)
		raise ArgumentError,
		"Malformed version number string #{version}" unless correct?(version)
		@version = version
	end
	
	##
	# Returns the text representation of the version
	#
	# return:: [String] version as string
	#
	def to_s
		@version
	end
	
	##
	# Checks if version string is valid format
	#
	# str:: [String] the version string
	# return:: [Boolean] true if the string format is correct, otherwise false
	#
	def correct?(str)
		/^#{NUM_RE}$/.match(str)
	end
	
	##
	# Convert version to integer array
	#
	# return:: [Array] list of integers
	#
	def to_ints
		@version.scan(/\d+/).map {|s| s.to_i}
	end
	
	##
	# Compares two versions
	#
	# other:: [Version or .to_ints] other version to compare to
	# return:: [Fixnum] -1, 0, 1
	#
	def <=>(other)
		rnums, vnums = to_ints, other.to_ints
		[rnums.size, vnums.size].max.times {|i|
		rnums[i] ||= 0
		vnums[i] ||= 0
		}
	
		begin
		r,v = rnums.shift, vnums.shift
		end until (r != v || rnums.empty?)
	
		return r <=> v
	end
end
